import java.util.List;
import java.util.Random;

/**
 * Jednoduchý model lišky.
 * Liška stárne, běhá, loví králíky a umírá.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class Liska
{
    // Charakteristiky společné pro všechny lišky (statické položky).
    
    // Věk, ve kterém se králík začíná množit.
    private static final int BREZI_VEK = 10;
    // Maximální stáří králíka.
    private static final int MAX_VEK = 150;
    // Pravděpodobnost, že králík zabřezne.
    private static final double PRAVDEPODOBNOST_ZABREZNUTI = 0.09;
    // Maximální počet mláďat.
    private static final int MAX_MLADAT = 3;
    // Kalorická hodnota jednoho králíka. Číslo určuje, kolikrát
    // může kroků může liška běhat, než musí sníst dalšího králíka.
    private static final int KRALIK_HODNOTA_JIDLA = 5;
    // Sdílený generátor náhodných čísel.
    private static final Random rand = new Random();
    
    // Individuální charakteristiky lišky (datové atributy).

    // Stáří lišky.
    private int vek;
    // zda je liska ziva.
    private boolean ziva;
    // Pozice lišky
    private Pozice pozice;
    // Zásoby jídla lišky, zvyšují se jezením králíků.
    private int zasobaJidla;

    /**
     * Vytvoří se liška. Liška se může narodit (věk je 0
     * a není hladová) či může mít náhodný věk a náhodnou zásobu
     * jídla (při generování počáteční situace)
     * 
     * @param nahodnyVek Pokud true, má liška náhodný věk a náhodnou zásobu jídla.
     */
    public Liska(boolean nahodnyVek) {
        vek = 0;
        ziva = true;
        if(nahodnyVek) {
            vek = rand.nextInt(MAX_VEK);
            zasobaJidla = rand.nextInt(KRALIK_HODNOTA_JIDLA);
        }
        else {
            // po narození nenecháme lišku zemřít hlady
            zasobaJidla = KRALIK_HODNOTA_JIDLA;
        }
    }
    
    /**
     * To co většinou liška dělá - loví králíky. Mimo to může porodit
     * mladé, zemřít hladem či zemřít stářím.
     * @param louka     aktuální louka.
     * @param novaLouka Nová louka.
     * @param narozeneLisky Seznam lišek, které se narodily.
     */
    public void lovit(Louka louka, List<Liska> narozeneLisky) {
        zvysVek();
        snezJidlo();
        if (ziva) {
            // Narozené lišky se umístí na sousední pozice.
            int mladat = porod();
            for(int b = 0; b < mladat; b++) {
                Liska novaLiska = new Liska(false);
                Pozice poz = louka.volnaSousedniPozice(pozice);
                if (poz != null) {  // pokud není místo, tak liška hned umře
                    narozeneLisky.add(novaLiska);
                    louka.vloz(novaLiska, poz);
                }
            }
            // Liška hledá v okolí jídlo (králíka).
            Pozice novaPozice = hledejJidlo(louka, pozice);
            if(novaPozice == null) {  // jídlo nenalezeno
                novaPozice = louka.volnaSousedniPozice(pozice);
            }
            
            if(novaPozice != null) {
                louka.presun(this, pozice, novaPozice);
            }
            else {
                // Pokud jsou všechny pozice obsazeny, tak umírá
                louka.odeberZvire(pozice);
                ziva = false;
            }
        }
        else {  // liška je mrtvá
            louka.odeberZvire(pozice);
        }
    }
    
    /**
     * Zvyší vek lišky. Pokud je moc stará, tak zemře.
     */
    private void zvysVek() {
        vek++;
        if(vek > MAX_VEK) {
            ziva = false;
        }
    }
    
    /**
     * Liška sní další zásoby jídla. Pokud již nic nemá, tak zemře.
     */
    private void snezJidlo() {
        zasobaJidla--;
        if(zasobaJidla < 0) {
            ziva = false;
        }
    }
    
    /**
     * Řekne lišce, že má v nejbližším okolí hledat králíka k snědku.
     * @param louka     Louka, na které hledá králíka.
     * @param pozice    Pozice na louce, na které se na začátku nachází.
     * @return          Pozice, na které našla králíka nebo null, pokud v okolí žádný není.
     */
    private Pozice hledejJidlo(Louka louka, Pozice pozice) {
        for (Pozice sousedni : louka.sousedniPozice(pozice)) {
            Object zvire = louka.getObjectAt(sousedni);
            if(zvire instanceof Kralik) {
                Kralik kralik = (Kralik) zvire;
                if(kralik.jeZivy()) { 
                    kralik.snezen();
                    louka.odeberZvire(sousedni);
                    zasobaJidla = KRALIK_HODNOTA_JIDLA;
                    return sousedni;
                }
            }
        }
        return null;
    }
        
    /**
     * Vrací počet mláďat lišky, pokud může zabřeznout.
     * @return Počet mláďat (může být i nula).
     */
    private int porod() {
        int  mladat = 0;
        if(vek >= BREZI_VEK && rand.nextDouble() <= PRAVDEPODOBNOST_ZABREZNUTI) {
            mladat = rand.nextInt(MAX_MLADAT) + 1;
        }
        return mladat;
    }
    
    /**
     * Zda je liška ještě živá.
     * @return true, pokud je liška ještě živá.
     */
    public boolean jeZiva() {
        return ziva;
    }

    /**
     * Nastavení pozice lišky.
     * @param location Pozice lišky.
     */
    public void setPozice(Pozice pozice) {
        this.pozice = pozice;
    }
}
