import java.util.Random;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.awt.Color;

/**
 * Jednoduchý simulátor louky s králíky a liškami.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class Simulator
{
    // Privátní statické položky nastavují parametry
    // simulace
    // Standardní šířka louky.
    private static final int DEFAULT_SIRKA = 50;
    // Standardní délka louky.
    private static final int DEFAULT_DELKA = 50;
    // Pravděpodobnost, že liška vznikne na konkrétní pozici.
    private static final double PRAVDEPODOBNOST_VYTVORENI_LISKY = 0.02;
    // Pravděpodobnost, že králík vznikne na konkrétní pozici.
    private static final double PRAVDEPODOBNOST_VYTVORENI_KRALIKA = 0.08;    

    // Seznamy zvířat na louce
    private List <Liska> lisky;
    private List <Kralik> kralici;
    // Seznamy narozených zvířat na louce
    private List <Kralik> narozeniKralici;
    private List <Liska> narozeneLisky;
    // Aktuální louka.
    private Louka louka;
    // Aktuální krok simulace.
    private int krok;
    // Grafické zobrazení simulace.
    private SimulatorView okno;
    
    /**
     * Vytvoření simulátoru se standardní délkou a šířkou.
     */
    public Simulator()  {
        this(DEFAULT_DELKA, DEFAULT_SIRKA);
    }
    
    /**
     * Vytvoření simulace louky se zadanou velikostí.
     * @param delka Délka louky. Musí být větší než nula.
     * @param sirka Šířka louky. Musí být větší než nula.
     */
    public Simulator(int delka, int sirka)
    {
        if(sirka <= 0 || delka <= 0) {
            System.out.println("Strany musí být větší než nula.");
            System.out.println("Použiji standardní hodnoty.");
            delka = DEFAULT_DELKA;
            sirka = DEFAULT_SIRKA;
        }
        kralici = new ArrayList<Kralik>();
        lisky = new ArrayList<Liska> ();
        narozeniKralici = new ArrayList<Kralik>();
        narozeneLisky = new ArrayList<Liska>();
        louka = new Louka(delka, sirka);

        // vytváří se instance pro zobrazení louky včetně nastavení barev
        okno = new SimulatorView(delka, sirka);
        okno.setColor(Liska.class, Color.red);
        okno.setColor(Kralik.class, Color.orange);
        
        // nastavení počátečního stavu
        internalReset();
    }
    
    /**
     * Spuštění simulace z aktuálního stavu po dlouhou dobu - po 500 období.
     */
    public void dlouhaSimulace() {
        simulace(500);
    }
    
    /**
     * Spustí simulaci z aktuální stavu na zadaný počet období (počet kroků).
     * Pokud na louce nezůstane žádné zvíře, skončí dříve.
     */
    public void simulace(int pocetKroku) {
        for(int krok = 0; krok < pocetKroku && okno.pokracovat(louka); krok++) {
            simulaceJedenKrok();
        }
    }
    
    /**
     * Odsimuluje jedno období od aktuálního stavu.
     * Projde se celá louka a aktualizuje stav každého králíka a každé lišky.
     */
    public void simulaceJedenKrok() {
        krok++;
        narozeniKralici.clear();
        narozeneLisky.clear();
        
        // nyní se zvířata projeví
        for (Liska liska : lisky) {
            if (liska.jeZiva()) {
                liska.lovit(louka, narozeneLisky);
            }
        }
        lisky.addAll(narozeneLisky);

        for (Kralik kralik : kralici) {
            if (kralik.jeZivy()) {
                kralik.behat(louka, narozeniKralici);
            }
        }
        kralici.addAll(narozeniKralici);
        // zobrazí se aktuální stav louky
        okno.zobrazStav(krok, louka);
    }
        
    /**
     * Nastaví simulaci do počáteční pozice (s náhodným
     * rozmístěním králíků a lišek).
     */
    public void reset() {
        internalReset();
    }

    /**
     * Nastaví simulaci do počáteční pozice (s náhodným
     * rozmístěním králíků a lišek) - v privátní metodě je to z toho důvodu,
     * že to volám i z konstruktoru.
     */    
    private void internalReset() {
        krok = 0;
        kralici.clear();
        lisky.clear();
        louka.clear();
        ozivit(louka);
        
        okno.zobrazStav(krok, louka);
    }
    
    /**
     * Oživí louku králíky a liškami.
     * @param louka Louka, která se má oživit.
     */
    private void ozivit(Louka louka) {
        Random rand = new Random();
        louka.clear();
        for (Pozice pozice : louka.seznamPozic()) {
                if(rand.nextDouble() <= PRAVDEPODOBNOST_VYTVORENI_LISKY) {
                    Liska liska = new Liska(true);
                    lisky.add(liska);
                    liska.setPozice(pozice);
                    louka.vloz(liska, pozice);
                }
                else if(rand.nextDouble() <= PRAVDEPODOBNOST_VYTVORENI_KRALIKA) {
                    Kralik kralik = new Kralik(true);
                    kralici.add(kralik);
                    kralik.setPozice(pozice);
                    louka.vloz(kralik, pozice);
                }
                // else pozice zůstane prázdná
        }
        Collections.shuffle(kralici);
        Collections.shuffle(lisky);
    }
}
