import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.HashMap;
import java.util.Map;

/**
 * Grafické zobrazení simulace louky.
 * Zobrazí se barevná mřížka, ve které každý čtvereček
 * barevně reprezentuje odpovídající pozici louky.
 * Pro jednotlivé druhy se barva určuje pomocí metody setColor.
 * 
 * @author David J. Barnes and Michael Kolling
 * @author Lubos Pavlicek
 * @version 2005.10.23
 */
public class SimulatorView extends JFrame {
    private static final long serialVersionUID=1;

    // Barva použitá pro neobsazené pole
    private static final Color EMPTY_COLOR = Color.green;

    // Barva použitá pro objekty, které nemají definovanou vlastní barvu
    private static final Color UNKNOWN_COLOR = Color.gray;

    private static final String STEP_PREFIX = "Krok: ";
    private static final String POPULATION_PREFIX = "Populace: ";
    private JLabel labKrok, labPopulace;
    private PanelLouky panelLouky;
    
    // Mapa použitá pro uložení barev přiřazených k jednotlivým druhům
    private Map<Class, Color> barvy;
    // Objekt, který spočítá a udržuje statistiky za louku
    private LoukaStats statistika;

    /**
     * Vytvoření mřížky se zadaným počtem řádků a sloupců.
     */
    public SimulatorView(int vyska, int sirka) {
        statistika = new LoukaStats();
        barvy = new HashMap<Class, Color>();

        setTitle("Simulace králíků a lišek");
        labKrok = new JLabel(STEP_PREFIX, JLabel.CENTER);
        labPopulace = new JLabel(POPULATION_PREFIX, JLabel.CENTER);
        
        setLocation(100, 50);
        
        panelLouky = new PanelLouky(vyska, sirka);

        add(labKrok, BorderLayout.NORTH);
        add(panelLouky, BorderLayout.CENTER);
        add(labPopulace, BorderLayout.SOUTH);
        pack();
        setVisible(true);
    }
    
    /**
     * Definuje barvu, která se použije pro zobrazení instancí příslušné třídy
     * @param animalClass   Třída, pro jejíchž instance se definuje barva
     * @param barva         Barva, která bude použita pro dannou třídu.
     */
    public void setColor(Class animalClass, Color barva) {
        barvy.put(animalClass, barva);
    }

    /**
     * @return Vrací barvu, která se použije pro zobrazení tříd
     */
    private Color najdiBarvu(Class animalClass) {
        Color barva = barvy.get(animalClass);
        if(barva == null) {
            return UNKNOWN_COLOR;
        }
        else {
            return barva;
        }
    }

    /**
     * Zobrazí aktuální stav louky
     * @param krok  Který je krok (zobrazí se v okně).
     * @param louka Louka, jejíž stav se zobrazí.
     */
    public void zobrazStav(int krok, Louka louka) {
        if(!isVisible()) {
            setVisible(true);
        }
            
        labKrok.setText(STEP_PREFIX + krok);
        statistika.reset();
        
        panelLouky.priprava();

        for(int radek = 0; radek < louka.getDelka(); radek++) {
            for(int sloupec = 0; sloupec < louka.getSirka(); sloupec++) {
                Object zvire = louka.getObjectAt(radek, sloupec);
                if(zvire != null) {
                    statistika.zvysCitac(zvire.getClass());
                    panelLouky.nakresliZnacku(radek, sloupec, najdiBarvu(zvire.getClass()));
                }
                else {
                    panelLouky.nakresliZnacku(radek, sloupec, EMPTY_COLOR);
                }
            }
        }
        statistika.spocteno();

        labPopulace.setText(POPULATION_PREFIX + statistika.getDetailyOPopulaci(louka));
        panelLouky.repaint();
    }
    
    /**
     * Zjišťuje, zda se má smysl ještě v simulaci pokračovat.
     * @return true Pokud je na louce více než jeden druh zvířete.
     */
    public boolean pokracovat(Louka louka) {
        return statistika.pokracovat(louka);
    }
    
    /**
     * Zobrazuje mřížku s jednotlivými políčky. Je to vnitřní
     * třída, která definuje vlastní komponentu uživatelského rozhraní.
     * Tato třída se používá pro zobrazení louky.
     * Pochopení třídy předpokládá dobré znalosti GUI v Javě.
     */
    private class PanelLouky extends JPanel {
        private static final long serialVersionUID=1;

        private static final int GRID_VIEW_SCALING_FACTOR = 8;
        
        private int pocetSloupcu, pocetRadku;
        private int xScale, yScale;
        private Dimension velikost;
        private Graphics g;
        private Image obrazLouky;

        /**
         * Nastaví počáteční stav.
         */
        public PanelLouky(int vyska, int sirka) {
            pocetRadku = vyska;
            pocetSloupcu = sirka;
            velikost = new Dimension(0, 0);
        }

        /**
         * Sdělí správci grafiky (GUI) svoji preferovanou velikost.
         */
        public Dimension getPreferredSize() {
            return new Dimension(pocetSloupcu * GRID_VIEW_SCALING_FACTOR,
                                 pocetRadku * GRID_VIEW_SCALING_FACTOR);
        }
        
        /**
         * Připraví prostředí pro nakreslení louky v novém kole.
         * Ošetřuje se zde situace, kdy se změní velikost panelu (uživatel
         * změní velikost celého okna) - musí se vypočítat nové koeficienty.
         */
        public void priprava() {
            if(! velikost.equals(getSize())) {  // když se velikost změní (též napoprvé)
                velikost = getSize();
                obrazLouky = panelLouky.createImage(velikost.width, velikost.height);
                g = obrazLouky.getGraphics();

                xScale = velikost.width / pocetSloupcu;
                if(xScale < 1) {
                    xScale = GRID_VIEW_SCALING_FACTOR;
                }
                yScale = velikost.height / pocetRadku;
                if(yScale < 1) {
                    yScale = GRID_VIEW_SCALING_FACTOR;
                }
            }
        }
        
        /**
         * Vykreslí zadané políčko v mřížce zadanou barvou.
         */
        public void nakresliZnacku(int x, int y, Color barva) {
            g.setColor(barva);
            g.fillRect(x * xScale, y * yScale, xScale-1, yScale-1);
        }

        /**
         * Překreslení louky na obrazovce - vnitřně vytvořený obrázek
         * se zkopíráje na obrazovku.
         */
        public void paintComponent(Graphics g) {
            if(obrazLouky != null) {
                Dimension aktualniVelikost = getSize();
                if(velikost.equals(aktualniVelikost)) {
                    g.drawImage(obrazLouky, 0, 0, null);
                }
                else {
                    g.drawImage(obrazLouky, 0, 0, aktualniVelikost.width, aktualniVelikost.height, null);
                }
            }
        }
    }
}
